/*
 * Copyright (c) 2009-2013, toby weston & tempus-fugit committers
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.code.tempusfugit.temporal;

import java.util.Random;

/**
 * Get a random {@link Duration} up to a specified amount. Use with {@link Introduce} to apply a random sleep up to a
 * given amount.
 *
 * @see {@link Introduce}
 */
public class UpTo {

    private final Duration duration;
    private final Random random = new Random();

    private UpTo(Duration duration) {
        this.duration = duration;
    }

    public static Duration millis(int duration) {
        return new UpTo(Duration.millis(duration)).duration();
    }

    public static UpTo upTo(Duration duration) {
        return new UpTo(duration);
    }

    public Duration duration() {
        return Duration.millis(random.nextInt((int) duration.inMillis() + 1));
    }
}
